/*
 * Copyright (C) 2008 Tony Wasserka
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 *
 */

#include <stdio.h>
#include <math.h>
#include <allegro.h>
#include "global.h"
#include "particles.h"
#include "bullet.h"

/* yes, this is a dirty hack.. */
bool kill_bullet=false;
int bkill_x, bkill_y;

CBullet::~CBullet()
{

}

void check_bcollide(BITMAP *terrain, int x, int y, int col)
{
    /* strip out the alpha channel */
    if(((long*)terrain->line[y])[x]&0x00FFFFFF) {
        kill_bullet=true;
        bkill_x=x;
        bkill_y=y;
    }
}

int CBullet::logic(BITMAP *terrain, CParticleManager *particleman, CPlayer *players)
{
    vy+=GRAVITY;
    x+=vx;
    y+=vy;
    if(x<2) return 1;
    if(y<2) return 1;
    if(x>SCREEN_W-2) return 1;
    if(y>SCREEN_H-2) return 1;

    /* Check wether we collided with anything on the terrain */
    do_line(terrain, (int)x, (int)y, (int)(x-vx), (int)(y-vy), 0, check_bcollide);

    if(kill_bullet) {
        kill_bullet=false;
        particleman->explode(terrain, bkill_x, bkill_y, vx, vy, sqrtf(vx*vx+vy*vy)*2, players);
        /* possible TODO: just use the middle of both colors */
        switch(rand()%4) {
        case 0: if(explosion1) play_sample(explosion1, 255, (int)bkill_x*255/SCREEN_W, 1000, 0); break;
        case 1: if(explosion2) play_sample(explosion2, 255, (int)bkill_x*255/SCREEN_W, 1000, 0); break;
        case 2: if(explosion3) play_sample(explosion3, 255, (int)bkill_x*255/SCREEN_W, 1000, 0); break;
        case 3: if(explosion4) play_sample(explosion4, 255, (int)bkill_x*255/SCREEN_W, 1000, 0); break;
        }
        return 1;
    }

    return 0;
}

void CBullet::draw(BITMAP *buffer)
{
    circlefill(buffer, (int)x, (int)y, 5, col);
}

CBulletManager::CBulletManager()
{
    root=NULL;
}

CBulletManager::~CBulletManager()
{
    while(root) remove(root);
}

void CBulletManager::logic(BITMAP *terrain, CParticleManager *particleman, CPlayer *players)
{
    CBullet *cur=root;

    while(cur) {
        if(cur->logic(terrain, particleman, players)) {
            CBullet *buf=cur->next;
            remove(cur);
            cur=buf;
        } else cur=cur->next;
    }
}

void CBulletManager::draw(BITMAP *buffer)
{
    CBullet *cur=root;

    while(cur) {
        cur->draw(buffer);
        cur=cur->next;
    }
}

void CBulletManager::attach(CBullet *bullet)
{
    if(!bullet) return;
    if(!root) {
        root=bullet;
        bullet->next=NULL;
        log("CBulletManager: Successfully attached bullet %p as root.\n", bullet);
    }
    else {
        bullet->next=root->next;
        root->next=bullet;
        log("CBulletManager: Successfully attached bullet %p.\n", bullet);
    }
}

void CBulletManager::remove(CBullet *bullet)
{
    CBullet *cur=root;

    if(bullet==root) {
        cur=root->next;
        log("CBulletManager: removed bullet %p.\n", bullet);
        delete root;
        root=cur;
        return;
    }

    while(cur && cur->next) {
        if(cur->next==bullet) {
            cur->next=bullet->next;
            log("CBulletManager: removed bullet %p.\n", bullet);
            delete bullet;
            return;
        }
        cur=cur->next;
    }
    log("CBulletManager: Strange, the bullet manager was requested to remove bullet %p, but it wasn't part of our bullet list!\n", bullet);
}
