/*
 * Copyright (C) 2008 Tony Wasserka
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 *
 */

#include <stdio.h>
#include <allegro.h>
#include "loadpng.h"
#include "global.h"
#include "game.h"
#include "particles.h"
#include "bullet.h"
#include "player.h"
#include "dialog.h"

/* I'd never though I'd really use sounds in this game, so this is once again way too hacky actually >_< */
SAMPLE *jump=NULL;
SAMPLE *explosion1=NULL;
SAMPLE *explosion2=NULL;
SAMPLE *explosion3=NULL;
SAMPLE *explosion4=NULL;
SAMPLE *ground1=NULL;
SAMPLE *ground2=NULL;
bool nodialogs=false;

CGame::CGame()
{
    terrain=NULL;
    particles=NULL;
    bullets=NULL;
    players=NULL;
    dialog=NULL;
    gametime=0;
    dead[0]=false;
    dead[1]=false;
    dead[2]=false;
    dead[3]=false;
}

CGame::~CGame()
{

}

int CGame::init()
{
    char buf[512];
    char name[512];
    SAMPLE *buffer;

    get_executable_name(buf, 512);
    replace_filename(buf, buf, "nodialogs", 512);
    if(exists(buf)) nodialogs=true;

    sprintf(name, filename1);
    replace_filename(buf, buf, name, 512);
    terrain=load_png(buf, NULL);
    if(!terrain) {
        sprintf(name, filename2);
        sprintf(name, "terrain%d.png", (rand()%5)+1);
        replace_filename(buf, buf, name, 512);
        terrain=load_png(buf, NULL);
        if(!terrain) {
            sprintf(name, "terrain%d.png", (rand()%5)+1);
            replace_filename(buf, buf, name, 512);
            terrain=load_png(buf, NULL);
            if(!terrain) {
                log("Couldn't load terrain bitmap file %s from %s!\n", name, buf);
                return 1;
            }
        }
    }

    for(int j=0;j<8;j++) {
        for(int k=0;k<SCREEN_W;k++) {
#define COL (255-16*(j+1))
            if(!(((long*)terrain->line[j])[k]&0x00FFFFFF)) ((long*)terrain->line[j])[k]=makecol(COL, COL, COL);
            if(!(((long*)terrain->line[SCREEN_H-j-1])[k]&0x00FFFFFF)) ((long*)terrain->line[SCREEN_H-j-1])[k]=makecol(COL, COL, COL);
            if(k<SCREEN_H) {
                if(!(((long*)terrain->line[k])[j]&0x00FFFFFF)) ((long*)terrain->line[k])[j]=makecol(COL, COL, COL);
                if(!(((long*)terrain->line[k])[SCREEN_W-j-1]&0x00FFFFFF)) ((long*)terrain->line[k])[SCREEN_W-j-1]=makecol(COL, COL, COL);
            }
        }
    }

    replace_filename(buf, buf, "jump.wav", 512);
    jump=load_wav(buf);
    replace_filename(buf, buf, "explosion1.wav", 512);
    explosion1=load_wav(buf);
    replace_filename(buf, buf, "explosion2.wav", 512);
    explosion2=load_wav(buf);
    replace_filename(buf, buf, "explosion3.wav", 512);
    explosion3=load_wav(buf);
    replace_filename(buf, buf, "explosion4.wav", 512);
    explosion4=load_wav(buf);
    replace_filename(buf, buf, "ground1.wav", 512);
    ground1=load_wav(buf);
    replace_filename(buf, buf, "ground2.wav", 512);
    ground2=load_wav(buf);

    particles=new CParticleManager;
    bullets=new CBulletManager;
    players=new CPlayer[4];

    for(int i=0;i<4;i++) {
        int px, py;
        do {
            px=rand()%SCREEN_W;
            py=rand()%SCREEN_H;
        } while (((long*)terrain->line[py])[px]&0x00FFFFFF || ((long*)terrain->line[py])[px+1]&0x00FFFFFF || ((long*)terrain->line[py])[px+2]&0x00FFFFFF ||
                 ((long*)terrain->line[py])[px-1]&0x00FFFFFF || ((long*)terrain->line[py])[px-2]&0x00FFFFFF);

        players[i].init(px, py);
    }

    players[0].set_controls(KEY_LEFT, KEY_RIGHT, KEY_UP, KEY_DOWN, KEY_ENTER, KEY_SPACE);
    players[1].set_controls(KEY_A, KEY_D, KEY_W, KEY_S, KEY_Q, KEY_F);
    players[2].toggle_ai(true);
    players[3].toggle_ai(true);

    return 0;
}

int CGame::exit()
{
    if(particles) delete particles;
    if(bullets) delete bullets;
    if(players) {
        for(int i=0;i<4;i++)
            players[i].exit();
        delete[] players;
    }
    if(dialog) delete dialog;

    return 0;
}

int CGame::logic()
{
    int players_dead=0;
    static int dead_counter=0;

    gametime++;

    for(int i=0;i<4;i++) {
        players[i].logic(terrain, bullets, players, i);
        if(players[i].health<=0) players_dead++;
    }

    particles->logic(terrain, players);
    bullets->logic(terrain, particles, players);

    if(gametime==100 && gametime < 1100 && !dialog && !nodialogs) dialog=new CGameDialog(1000, "Hi mate", "Guess you're astonished to see such a shitty game, what?", "However, please take note, that its coder spent only 2 days on coding it.", "This game was programmed for the TINS Is Not Speedhack competition in 2008", "Have as much fun as you're able to ;-)");
    if(gametime==2000 && gametime < 3000 && !dialog && !nodialogs) dialog=new CGameDialog(1000, "Hello again.", "If you ever wondered why this game is so boring,", "its creator was a dumbass and has no clue of whatsoever.", "I really wonder how he even finished this game", "with his \"small brain\" ;-)");
    if(gametime>=4000 && gametime < 5000 && !dialog && !nodialogs) dialog=new CGameDialog(1000, "Yet another message for you...", "If you think this game is boring", "try to play it with a friend!", "This game is much funnier when you play it against another human ;-)", "At least if the bots are too easy for you :P");
    if(gametime>=6000 && gametime < 7000 && !dialog && !nodialogs) dialog=new CGameDialog(1000, "Quick note about physics:", "I guess you already wondered about the strange physics here.", "They are (well, more or less) intentional, because the can have", "a positive impact on gameplay, as it allows some tricks", "you could not do in normal physics.");
    if(gametime>=8000 && gametime < 9000 && !dialog && !nodialogs) dialog=new CGameDialog(1000, "Make sure you check out my homepage, too", "http://tonwas.freefronthost.com", "Especially Snake 256 is worth a try!", "", "");
    if(gametime>=10000 && gametime < 11000 && !dialog && !nodialogs) dialog=new CGameDialog(1000, "Okay, I guess you don't want to read this anymore :D", "Press F12 NOW if you don't want to be disturbed by this again", "However, if you want to read it once again, you can turn it on with F12 if you want ;-)", "", "");

    if(gametime>=10000 && gametime < 11000 && !nodialogs && key[KEY_F12]) {
        char buf[512];
        FILE *nd;

        get_executable_name(buf, 512);
        replace_filename(buf, buf, "nodialogs", 512);

        nd=fopen(buf, "w");
        if(nd) {
            fprintf(nd, ".\n");
            fclose(nd);
        }
        if(dialog) { delete dialog; dialog=NULL; };

        nodialogs=true;
        while(key[KEY_F12]);
    } else if (nodialogs && key[KEY_F12]) {
        char buf[512];

        get_executable_name(buf, 512);
        replace_filename(buf, buf, "nodialogs", 512);
        delete_file(buf);

        nodialogs=false;
        while(key[KEY_F12]);
    }

    if(dialog) if(dialog->logic()) {
        delete dialog;
        dialog=NULL;
    }

    if(!dialog) {
        if(!dead[0] && players[0].health<=0) {
            if(!nodialogs) dialog=new CGameDialog(1000, "PLAYER 1 IS DEAD", "Dude, next time you should better watch out!", "Don't waste you time with doing nothing, dumbass", "Shoot your enemies before they do so, idiot!", "Good luck next time ;-)");
            dead[0]=true;
        } else if(!dead[1] && players[1].health<=0) {
            if(!nodialogs) dialog=new CGameDialog(1000, "PLAYER 2 IS DEAD", "Man, you're so lame, even my grandma would be better in this game!", "Open your damn eyes and don't just stand there getting shot!", "When watching your gameplay I was thinking YOU WERE A BOT, noob!", "I hope you learn you lesson next time!");
            dead[1]=true;
        } else if(!dead[2] && players[2].health<=0) {
            if(!nodialogs) dialog=new CGameDialog(1000, "PLAYER 3 IS DEAD", "C'mon, I've seen bots in Snake 256 following their tale until they die,", "but what YOU have played in THIS game was even worse!", "C'mon why the hell are you bastard just standing still doing nothing but shoot in the wrong direction??", "The guy who coded the AI must have been a real dumbass!");
            dead[2]=true;
        } else if(!dead[3] && players[3].health<=0) {
            if(!nodialogs) dialog=new CGameDialog(1000, "PLAYER 4 IS DEAD", "But I actually do not care at all.", "Our fucking coder was too lazy too insert some creative text here,", "so I won't tell you a nice comment and whatever you want to do now", "I don't care, STFU.");
            dead[3]=true;
        }
    }

    if(players_dead==3 && !dialog && dead_counter<3000) {
        dialog=new CGameDialog(2500, "Only one player is still alive!", "Congrats to you brave guy who was never afraid to risk your life for a glory battle!", "Guess you better restart the game and play another round now...", "Prepare for something fun!", "But the idiotic losers better don't play such a shit this time!");
    }
    if(players_dead>=3) dead_counter++;
    if(dead_counter>3000) {
        change_state(NULL);
    }

    if(key[KEY_ESC]) change_state(NULL);

    return 0;
}

int CGame::draw(BITMAP *buffer)
{
    int x, y;

    /* Blitting the whole terrain bitmap using X drivers is way too slow, so we're doing it the fast way */
    for(y=0;y<SCREEN_H;y++) {
        long *buffery=(long*)buffer->line[y];
        long *terrainy=(long*)terrain->line[y];
        for(x=0;x<SCREEN_W;x++) {
            if(terrainy[x]&0x00FFFFFF) /* If the pixel is not black */
                buffery[x]=terrainy[x];
        }
    }

    for(int i=0;i<4;i++)
        players[i].draw(buffer);

    particles->draw(buffer);
    bullets->draw(buffer);

    if(dialog) dialog->draw(buffer);

    return 0;
}
