/* Things which any source file wants to know */
/*
 * Copyright (C) 2008 Tony Wasserka
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 *
 */

#define GRAVITY 0.0981f

class CGameState
{
public:
    CGameState() {};
    ~CGameState() {};

    virtual int init() {return 0;};
    virtual int exit(){return 0;};
    virtual int logic(){return 0;};
    virtual int draw(BITMAP *buffer){return 0;};
};

class CMenu : public CGameState
{
public:
    CMenu();
    ~CMenu();

    int init();
    int exit();
    int logic();
    int draw(BITMAP *buffer);
private:
    int menutime;
};

class CParticleManager;
class CPlayer;
class CBulletManager;
class CBullet;
class CGameDialog;
class CGame : public CGameState
{
public:
    CGame();
    ~CGame();

    int init();
    int exit();
    int logic();
    int draw(BITMAP *buffer);
private:
    BITMAP *terrain;
    CParticleManager *particles;
    CBulletManager *bullets;
    CPlayer *players;
    CGameDialog *dialog;
    int gametime;
    bool dead[4];
};

void log(char *str, ...);
void change_state(CGameState *new_state);
void shoot_bullet(CBullet *bullet);
BITMAP *load_png(AL_CONST char *filename, RGB *pal); /* Thanks to Peter Wang for this function! */

extern SAMPLE *jump; /* I tried to avoid global vars at my best, and succeeded until I wanted to add sounds :*/
extern SAMPLE *explosion1;
extern SAMPLE *explosion2;
extern SAMPLE *explosion3;
extern SAMPLE *explosion4;
extern SAMPLE *ground1;
extern SAMPLE *ground2;
extern char filename1[256];
extern char filename2[256];

