/*
 * Copyright (C) 2008 Tony Wasserka
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 *
 */

#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <time.h>
#include <allegro.h>
#include "global.h"
#include "main.h"

CLogger logger;
CApplication app;
int timer=0, lastfps=0, fps=0;
bool fullscreen=false;
char filename1[256];
char filename2[256];

CApplication::CApplication()
{
    buffer=NULL;
    state=NULL;
}

CApplication::~CApplication()
{
    if(state) delete state;
}

int CApplication::init()
{
    srand(time(NULL));

    buffer=create_bitmap(SCREEN_W, SCREEN_H);
    if(buffer==NULL) {
        log("app.init: Failed to create bitmap buffer!\n");
        return INIT_FAIL;
    }
    clear_bitmap(buffer);

    change_state(new CMenu);

    return INIT_SUCCESS;
}

void CApplication::exit()
{
    if(buffer) destroy_bitmap(buffer);
    if(state) state->exit();
}

int CApplication::game_loop()
{
    while(state) {
        while(timer>0) {
            if(state) state->logic();
            timer--;
        }
        if(state) state->draw(buffer);
//        textprintf_ex(buffer, font, 0, 10, makecol(255, 255, 255), -1, "FPS: %d", lastfps);
        blit(buffer, screen, 0, 0, 0, 0, SCREEN_W, SCREEN_H);
        clear_bitmap(buffer);
        fps++;
    }
    return ENDLOOP_QUIT;
}

void CApplication::change_state(CGameState *new_state)
{
    if(state) {
        state->exit();
        delete state;
    }
    state=new_state;
    if(state) {
        if(state->init()) {
            allegro_message("Failed to change game state!\n");
            log("Aborting because switching the game state failed.\n");
            delete state;
            state=NULL;
        }
    }
}

CLogger::CLogger()
{
    file=fopen("log.txt", "w");
}

CLogger::~CLogger()
{
    fclose(file);
}

void CLogger::log(char *str)
{
    fprintf(file, str);
}

void log(char *str, ...)
{
    char buf[256]; /* Hope that's enough ;-) */
    va_list ap;

    /* Uhm... yeah, we probably could simplify these lines into 3 or so,
       but that would require some in-depth understanding of the va functions, but I don't have time for that :D */
    va_start(ap, str);
    uvszprintf(buf, 256, str, ap); /* Thanks to allegro.c's allegro_message! */
    va_end(ap);

    logger.log(buf);
}

void change_state(CGameState *new_state)
{
    app.change_state(new_state);
}

void timer_func()
{
    timer++;
}

void fps_func()
{
    lastfps=fps;
    fps=0;
}

int main(int argc, char *argv[])
{
    char buf[256];
    log("Started application, log file created.\n");

    log("Calling allegro_init(): ");
    if(allegro_init()!=0) {
        printf("Mr. Wombad says: \"Dude, you couldn't even manage to initialize Allegro!\n Sorry man, but you can't play this game.\n\"");
        log("FAIL! Quitting...\n");
        exit(0);
    } else log("Success!\n");
    install_keyboard();
    install_timer();
    install_sound(DIGI_AUTODETECT, MIDI_NONE, NULL);
    set_color_depth(32);

strcpy(filename1, argv[0]);
if(argc>1)
    strcpy(filename2, argv[1]);

    get_executable_name(buf, 512);
    replace_filename(buf, buf, "fullscreen", 512);
    if(exists(buf))
        set_gfx_mode(GFX_AUTODETECT, 800, 600, 0, 0);
    else
        set_gfx_mode(GFX_AUTODETECT_WINDOWED, 800, 600, 0, 0);

    install_int_ex(timer_func, BPS_TO_TIMER(100));
    install_int_ex(fps_func, SECS_TO_TIMER(1));

    log("Initializing application: ");
    if(app.init()!=INIT_SUCCESS) {
        allegro_message("Mr. Wombad says: \"Sorry man, but I couldn't initialize the game correctly.\n Guess you can't play this game.\n\"");
        log("FAIL! Quitting...\n");
        allegro_exit();
        exit(0);
    } else {
        log("Success!\n");
        log("Entering main loop now:\n");
        app.game_loop();
    }

    log("Game loop finished. Calling app.exit now.\n");
    app.exit();
    log("Seems like all things went well. Closing log file now.\n");

    return 0;
}
