/*
 * Copyright (C) 2008 Tony Wasserka
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 *
 */

#include <stdio.h>
#include <math.h>
#include <allegro.h>
#include "global.h"
#include "menu.h"

#define NUM_STRINGS 15
#define DELAY 600

bool skippable=false;
int page=0;

CMenu::CMenu()
{
    menutime=0;
}

CMenu::~CMenu()
{

}

int CMenu::init()
{
    char buf[512];

    get_executable_name(buf, 512);
    replace_filename(buf, buf, "skip", 512);

    if(exists(buf)) skippable=true;

    return 0;
}

int CMenu::exit()
{
    FILE *fs;
    char buf[512];

    get_executable_name(buf, 512);
    replace_filename(buf, buf, "skip", 512);

    if(!exists(buf) && menutime>NUM_STRINGS*DELAY && page==1) {
        fs=fopen(buf, "w");
        if(fs) {
            fprintf(fs, ".\n");
            fclose(fs);
        }
    }
    return 0;
}

int CMenu::logic()
{
    menutime++;

    if(key[KEY_ENTER]) {
        if(menutime>NUM_STRINGS*DELAY && page==0) {
            page=1;
            while(key[KEY_ENTER]);
            menutime=0;
        } else if(skippable || (menutime > NUM_STRINGS*DELAY && page==1)) {
            log("Enter was pressed in the menu, changing game state to \"Game\"\n");
            change_state(new CGame);
        }
    }

    if(key[KEY_F12]) {
        char buf[512];

        get_executable_name(buf, 512);
        replace_filename(buf, buf, "fullscreen", 512);

        if(exists(buf)) {
            set_gfx_mode(GFX_AUTODETECT_WINDOWED, 800, 600, 0, 0);
            delete_file(buf);
        } else {
            FILE *fs;

            set_gfx_mode(GFX_AUTODETECT, 800, 600, 0, 0);
            fs=fopen(buf, "w");
            if(fs) {
                fprintf(fs, ".\n");
                fclose(fs);
            }
        }
        while(key[KEY_F12]);
        menutime=0;
    }

    if(key[KEY_ESC]) change_state(NULL);

    return 0;
}

int CMenu::draw(BITMAP *buffer)
{
    const char strings[NUM_STRINGS][512]=
    {
    "Welcome to The Road to World Domination: Made simple",
    "Please take a few seconds to read some notes about this.",
    "This game was made within less than 72 hours and thus is not",
    "of such a quality that you may or may not be used to from comercial games.",
    "What I wanted to achieve with this game was to yet again learn how to",
    "code a game from scratch without doing much planning beforehand.",
    "I had one specific goal when I started to program this game:",
    "I wanted to do some magic stuff with completely destructable terrain,",
    "(if you ever played a game of the Worms series you know what I mean)",
    "but I wanted to go a step further and link the whole idea with particle systems",
    "and make the \"landscape pixels\" fly through the air and rejoin the",
    "area when they hit the ground. This goal is something which I accomplished",
    "pretty good in my opinion.",
    "--------------------------------------------------------",
    "---------------------- Press Enter ---------------------",
    };

    const char strings1[NUM_STRINGS][512]=
    {
    "About this game and how to play:",
    "You are one of the virtual 2D spheres (read readme.txt for the whole story)",
    "and your goal is to destroy the other 3 virtual 2D spheres.",
    "Control yourself either using the arrow keys, enter and space",
    "or via A/W/D/S/Q/F. Players 3 and 4 are computer enemies,",
    "The strange behavior of the physics engine is intentional",
    "This intro can be skipped next time you start the game.",
    "--------------------------------------------------------",
    "",
    "One last thing I want to tell you:",
    "Visit my homepage at http://tonwas.freefronthost.com",
    "and make sure you have played Snake 256 ;-)",
    "--------------------------------------------------------",
    "",
    "Press Enter to start playing now ;-)",
    };

    if(page==0) {
        for(int i=0;i<NUM_STRINGS;i++)
            if(menutime>i*DELAY) {
                char buf[512];
                strncpy(buf, strings[i], MIN(512, MIN((menutime-i*DELAY)/10, strlen(strings[i]))));
                buf[MIN(512, MIN((menutime-i*DELAY)/10, strlen(strings[i])))]='\0';
                textprintf_centre_ex(buffer, font, SCREEN_W/2, 10*i+50, makecol(MIN(menutime-i*DELAY, 255), MIN(menutime-i*DELAY, 255), MIN(menutime-i*DELAY, 255)), -1, buf);
            }
    } else if(page==1) {
        for(int i=0;i<NUM_STRINGS;i++)
            if(menutime>i*DELAY) {
                char buf[512];
                strncpy(buf, strings1[i], MIN(512, MIN((menutime-i*DELAY)/10, strlen(strings1[i]))));
                buf[MIN(512, MIN((menutime-i*DELAY)/10, strlen(strings1[i])))]='\0';
                textprintf_centre_ex(buffer, font, SCREEN_W/2, 10*i+50, makecol(MIN(menutime-i*DELAY, 255), MIN(menutime-i*DELAY, 255), MIN(menutime-i*DELAY, 255)), -1, buf);
            }
    }

    if(menutime<1000) {
        #define COL (MAX(255-menutime*255/1000, 0))
        textprintf_ex(buffer, font, 0, 0, makecol(COL, COL, COL), -1, "Press F12 to toggle fullscreen gaming");
    }

    return 0;
}
