/*
 * Copyright (C) 2008 Tony Wasserka
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 *
 */

#include <math.h>
#include <allegro.h>
#include "global.h"
#include "particles.h"
#include "player.h"

/* yes, this is a dirty hack.. */
bool kill_particle=false;
int kill_x, kill_y;

CParticle::~CParticle()
{

}

void check_collide(BITMAP *terrain, int x, int y, int col)
{
    /* strip out the alpha channel */
    if(((long*)terrain->line[y])[x]&0x00FFFFFF) {
        kill_particle=true;
        kill_x=x;
        kill_y=y;
    }
}

int CParticle::logic(BITMAP *terrain, CPlayer *players)
{
    vy+=GRAVITY;
    x+=vx;
    y+=vy;
    if(x<5) return 1;
    if(y<5) return 1;
    if(x>SCREEN_W-5) return 1;
    if(y>SCREEN_H-5) return 1;

    /* Check wether we collided with anything on the terrain */
    do_line(terrain, (int)x, (int)y, (int)(x-vx), (int)(y-vy), 0, check_collide);
    if(kill_particle) {
        kill_particle=false;

        /* possible TODO: just use the middle of both colors */
        ((long*)terrain->line[kill_y-(vy>0.5f*ABS(vx))+(vy<-0.5f*ABS(vx))])[kill_x-(vx>0.5f*ABS(vy))+(vx<-0.5f*ABS(vy))]=col;

        return 1;
    }

    return 0;
}

void CParticle::draw(BITMAP *buffer)
{
    circlefill(buffer, (int)x, (int)y, 1, col);
}

CParticleManager::CParticleManager()
{
    root=NULL;
}

CParticleManager::~CParticleManager()
{
    while(root) remove(root);
}

void CParticleManager::logic(BITMAP *terrain, CPlayer *players)
{
    CParticle *cur=root;

    while(cur) {
        if(cur->logic(terrain, players)) {
            CParticle *buf=cur->next;
            remove(cur);
            cur=buf;
        } else cur=cur->next;
    }
}

void CParticleManager::draw(BITMAP *buffer)
{
    CParticle *cur=root;

    while(cur) {
        cur->draw(buffer);
        cur=cur->next;
    }
}

#define LENSQ(x, y) ((x)*(x)+(y)*(y))

void CParticleManager::explode(BITMAP *terrain, int x, int y, float xv, float yv, float strength, CPlayer *players)
{
    for(int xc=x-(int)strength;xc<x+(int)strength;xc++) {
        for(int yc=y-(int)strength;yc<y+(int)strength;yc++) {
            if(yc > 0 && xc > 0 && yc < SCREEN_H && xc < SCREEN_W)
                if(((long*)terrain->line[yc])[xc]&0x00FFFFFF) {
                    if((xc-x)*(xc-x)+(yc-y)*(yc-y)<=strength*strength) {
                        attach(new CParticle(xc, yc, (rand()%1000)/400.f-1.25f, (yv>0) ? (-(rand()%1000)/400.f-2.f) : ((rand()%1000)/400.f), ((long*)terrain->line[yc])[xc]));
                        if(xc > 10 && yc > 10 && xc < SCREEN_W-10 && yc < SCREEN_H-10) ((long*)terrain->line[yc])[xc]=0;
                }
            }
        }
    }
    for(int i=0;i<4;i++) {
        if(LENSQ(players[i].x-(float)x, players[i].y-(float)y) < strength*strength*150) {
            int dmg=(int)(100.f/sqrtf(LENSQ(players[i].x-x, players[i].y-y)));
            if(dmg >= 1 && players[i].health>0) {
                log("Player %d gets %d damage", i, dmg);
                players[i].hit((dmg<100) ? dmg : 100);
                players[i].dmg=(dmg<100) ? dmg : 100;
                players[i].dmgcount=100;
            }
        }
    }
}

void CParticleManager::attach(CParticle *particle)
{
    if(!particle) return;
    if(!root) {
        root=particle;
        log("CParticleManager: Successfully attached particle %p as root.\n", particle);
    }
    else {
        particle->next=root->next;
        root->next=particle;
        log("CParticleManager: Successfully attached particle %p.\n", particle);
    }
}

void CParticleManager::remove(CParticle *particle)
{
    CParticle *cur=root;

    if(particle==root) {
        cur=root->next;
        log("CParticleManager: removed particle %p.\n", particle);
        delete root;
        root=cur;
        return;
    }

    while(cur && cur->next) {
        if(cur->next==particle) {
            cur->next=particle->next;
            log("CParticleManager: removed particle %p.\n", particle);
            delete particle;
            return;
        }
        cur=cur->next;
    }
    log("CParticleManager: Strange, the particle manager was requested to remove particle %p, but it wasn't part of our particle list!\n", particle);
}
