/*
 * Copyright (C) 2008 Tony Wasserka
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 *
 */

#include <stdio.h>
#include <math.h>
#include <allegro.h>
#include "global.h"
#include "bullet.h"
#include "player.h"

CPlayer::CPlayer()
{

}

CPlayer::~CPlayer()
{

}

void CPlayer::init(int px, int py)
{
    x=px;
    y=py;
    xv=0;
    yv=0;
    dir=(rand()%2)*2-1; /* -1=left; +1=right; */
    angle=(rand()%314)/100.f-1.57;
    jumping=true;
    power=0;
    reload=0;
    health=100;
    dmg=0;
    dmgcount=0;
    ai=false;
    replay=0;
    replayground=0;
}

void CPlayer::exit()
{

}

void CPlayer::set_controls(int left, int right, int up, int down, int jump, int shot)
{
    keyleft=left;
    keyright=right;
    keyup=up;
    keydown=down;
    keyjump=jump;
    keyshot=shot;
}

/* Dialog: "use [...] keys to control your character" */
void CPlayer::logic(BITMAP *terrain, CBulletManager *bulletman, CPlayer *other, int own_number)
{
    if(health > 0) { /* No intendation for this one, no time ;-) */
    if(!ai) {
        if(key[keyleft] && !jumping && !shooting) {
            xv=-2.f;
            dir=-1;
        }
        if(key[keyright] && !jumping && !shooting) {
            xv=2.f;
            dir=1;
        }
        if(key[keyup] && !jumping) angle-=0.05f;
        if(key[keydown] && !jumping) angle+=0.05f;
        if((!key[keyleft] && !key[keyright] && !jumping) || shooting) xv=0;
        if(key[keyjump] && !jumping && !shooting) {
            if(!replay) {
                play_sample(jump, 127, (int)x*255/SCREEN_W, 1000, 0);
                replay = 20;
            }
            jumping=true;
            xv=1.5f*dir;
            yv=-2.5;
        }
        if(!key[keyshot] && shooting!=1) shooting=0;
        if(key[keyshot] && !jumping && !shooting && !reload) {
            shooting=1;
            power=0;
        }
        if(key[keyshot] && !jumping && shooting==1) {
            power++;
        }
        if((!key[keyshot] && shooting==1) || power>210) {
            shooting=2;
            bulletman->attach(new CBullet(x, y, cosf(angle)*dir*log(power*2), sinf(angle)*log(power*2), makecol(255, 255, 255)));
            power=0;
            reload=200;
        }
    } else { /* Some AI */
        /* Our bots are too lazy to move, so we just leave that part away, but tell nobody :P */
        /* Find out who's nearest */
#define LENSQ(x, y) ((x)*(x)+(y)*(y))
        CPlayer *nearest=NULL;
        for(int i=0;i<4;i++) {
            if(i!=own_number) {
                if(nearest==NULL && other[i].health>0) {
                    nearest=&other[i];
                } else if(nearest==NULL) {
                    continue;
                }else if(LENSQ(x-other[i].x, y-other[i].y) < LENSQ(x-nearest->x, y-nearest->y) && other[i].health>0) {
                    nearest=&other[i];
                }
            }
        }

        if(nearest) {
        /* Now find the "perfect" angle to them */
        /* The nice thing about this is that I don't need to care about "emulated" mistakes as this approach is wrong anyways :D */
            float wanted_angle=atan2f(nearest->y-y, nearest->x-x);
            if(dir==-1 && (wanted_angle >  1.57f || wanted_angle < -1.57f)) {
                wanted_angle=SGN(wanted_angle)*3.14f-wanted_angle;
            } else if(dir==-1 && !(wanted_angle >  1.57f || wanted_angle < -1.57f)) {
                dir=1;
            } else if(dir==1 && (wanted_angle >  1.57f || wanted_angle < -1.57f)) {
                dir=-1;
                wanted_angle=3.14f-wanted_angle;
            }

            int xdist=ABS((int)x-(int)nearest->x);
            int ydist=ABS((int)y-(int)nearest->y);

            wanted_angle=wanted_angle-0.6f*((float)xdist/400.f);
            if(wanted_angle < -1.57) wanted_angle=-1.5f;

            if(angle>wanted_angle && !jumping) angle-=0.05f;
            if(angle<wanted_angle && !jumping) angle+=0.05f;

           int wanted_power=(int)(sqrtf(LENSQ(xdist, ydist)) / 350.f * 200.f);

            if(shooting!=1) shooting=0;
            if(ABS(wanted_angle-angle)<0.1f && !jumping && !shooting && !reload) {
                shooting=1;
                power=0;
            }
            if(power<wanted_power && !jumping && shooting==1) {
                power++;
            }
            if(power>=wanted_power || power>210) {
                shooting=2;
                bulletman->attach(new CBullet(x, y, cosf(angle)*dir*log(power*2), sinf(angle)*log(power*2), makecol(255, 255, 255)));
                power=0;
                reload=200;
            }
        }
    }
    x+=xv;
    y+=yv;
    if(reload) reload--;

    if(x<5) x=5;
    if(x>SCREEN_W-6) x=SCREEN_W-6;
    if(y<5) y=5;
    if(y>SCREEN_H-6) y=SCREEN_H-6;
    if(angle>1.57f) angle=1.57f;
    if(angle<-1.57f) angle=-1.57f;

    jumping=true;
    if(yv>=0) {
        for(int i=(int)(y-yv);i<y+5;i++) {
            if(((long*)terrain->line[i])[(int)x]&0x00FFFFFF || ((long*)terrain->line[i])[(int)x+1]&0x00FFFFFF || ((long*)terrain->line[i])[(int)x+2]&0x00FFFFFF ||
               ((long*)terrain->line[i])[(int)x-1]&0x00FFFFFF || ((long*)terrain->line[i])[(int)x-2]&0x00FFFFFF) {
                if(yv>2.1 && !replayground) {
                    switch(rand()%2) {
                    case 0: play_sample(ground1, 127, (int)x*255/SCREEN_W, 1000, 0); break;
                    case 1: play_sample(ground2, 127, (int)x*255/SCREEN_W, 1000, 0); break;
                    }
                    replayground=10;
                }
                jumping=false;
                yv=2; /* Keep us on the floor please */
                y=i-5;
                break;
            }
        }
    } else if(jumping) {
        for(int i=(int)y;i<y-yv+5;i++) {
            if(((long*)terrain->line[i])[(int)x]&0x00FFFFFF || ((long*)terrain->line[i])[(int)x+1]&0x00FFFFFF || ((long*)terrain->line[i])[(int)x+2]&0x00FFFFFF ||
               ((long*)terrain->line[i])[(int)x-1]&0x00FFFFFF || ((long*)terrain->line[i])[(int)x-2]&0x00FFFFFF) {
                if(yv<-1 && !replayground) {
                    switch(rand()%2) {
                    case 0: play_sample(ground1, 127, (int)x*255/SCREEN_W, 1000, 0); break;
                    case 1: play_sample(ground2, 127, (int)x*255/SCREEN_W, 1000, 0); break;
                    }
                    replayground=10;
                }
                y=i+5;
                yv=ABS(yv);
                break;
            }
        }
    }
    }

    if(jumping) yv+=GRAVITY;
    if(dmgcount) dmgcount--;

    /* Just to make sure... */
    if(x<5) x=5;
    if(x>SCREEN_W-6) x=SCREEN_W-6;
    if(y<5) y=5;
    if(y>SCREEN_H-6) y=SCREEN_H-6;

    if(replay) replay--;
    if(replayground) replayground--;
}

void CPlayer::draw(BITMAP *buffer)
{
    if(health>0) {
        circle(buffer, (int)x, (int)y, 5, makecol(255-((health>0) * health)*255/100, 64, ((health>0) * health)*255/100));
        line(buffer, (int)x, (int)y, (int)(x+cosf(angle)*5*dir), (int)(y+sinf(angle)*5), makecol(255-((health>0) * health)*255/100, 64, ((health>0) * health)*255/100));
    } else if(dmgcount) {
        circle(buffer, (int)x, (int)y, 5, makecol(255*dmgcount/100, 64*dmgcount/100, 0));
        line(buffer, (int)x, (int)y, (int)(x+cosf(angle)*5*dir), (int)(y+sinf(angle)*5), makecol(255*dmgcount/100, 64*dmgcount/100, 0));
    }
    if(shooting==1 && health > 0) {
        rect(buffer, (int)x-12, (int)y-15, (int)x+12, (int)y-10, makecol(128, 128, 128));
        rectfill(buffer, (int)x-10, (int)y-13, (int)(x-10.f+(float)power/10.f), (int)y-12, makecol(128, 0, 0));
    }
    if(dmgcount) {
        textprintf_centre_ex(buffer, font, (int)x, (int)y-20, makecol((255-((health>0) * health)*255/100)*dmgcount/100, 128*dmgcount/100, ((health>0) * health)*255/100*dmgcount/100), -1, "%d!", dmg);
    }
}

void CPlayer::hit(int strength)
{
    health-=strength;
/* TODO: activate some effects here */
}

void CPlayer::toggle_ai(bool use_ai)
{
    ai=use_ai;
}
